﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace DoublePendulum
{
	public partial class MainPage : UserControl
	{
		// 画面の位置
		const int DISP_WIDTH  = 400;
		const int DISP_HEIGHT = 400;

		int width, height;
		int cx, cy;

		// 各種パラメータ
		const int R_BALL = 12;   // ボールの直径
		double x1, y1, x2, y2; // ボールの座標
		double A, B, M1, M2, R1, R2;
		double[] TH = new double[2];
		double[] OM = new double[2];
		const double dt = 0.001; //時間刻み

		// 描画オブジェクト
		MyCircle[] balls = new MyCircle[2];
		Line[] lines = new Line[2];

		public MainPage()
		{
			InitializeComponent();

			// 画面サイズを設定
			width  = DISP_WIDTH;
			height = DISP_HEIGHT;
			cx = width / 2;
			cy = height / 2;

			initObjects();
			CompositionTarget.Rendering += new EventHandler(MainHandler);
		}

		// 変数の初期化
		private void initVars()
		{
			Random rnd = new System.Random();

			x1 = rnd.Next(-750, 750) / 1000.0;
			y1 = rnd.Next(-500, 750) / 1000.0;    //上側に出やすくする
			x2 = rnd.Next(-750, 750) / 1000.0;
			y2 = rnd.Next(-500, 750) / 1000.0;    //上側に出やすくする
			
			// 各種パラメータの初期化
			M1 = 1.0;   //１個目の重さ
			M2 = 2.0;   //２個目の重さ

			R1 = Math.Sqrt(x1 * x1 + y1 * y1);  //１個目までの長さ
			R2 = Math.Sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));    //２個目までの長さ
			TH[0] = Math.Atan2(x1, -y1);  //１個目の角度
			TH[1] = Math.Atan2(x2 - x1, y1 - y2); //２個目の角度
			OM[0] = 0;
			OM[1] = 0;

			A = (M1 + M2) * R1 / M2 / R2;
			B = R2 / R1;
		}

		// オブジェクトの初期化
        private void initObjects()
        {
			initVars();

			// 2本の線を設置
			lines[0] = new Line();
			lines[0].X1 = dispX(0.0);
			lines[0].Y1 = dispY(0.0);
			lines[0].X2 = dispX(x1);
			lines[0].Y2 = dispY(y1);
			lines[0].Stroke = new SolidColorBrush(Colors.Black);
			this.LayoutRoot.Children.Add(lines[0]);

			lines[1] = new Line();
			lines[1].X1 = dispX(x1);
			lines[1].Y1 = dispY(y1);
			lines[1].X2 = dispX(x2);
			lines[1].Y2 = dispY(y2);
			lines[1].Stroke = new SolidColorBrush(Colors.Black);
			this.LayoutRoot.Children.Add(lines[1]);

			// 2個のボールを設置
			balls[0] = new MyCircle(dispX(x1), dispY(y1), R_BALL);
			this.LayoutRoot.Children.Add( balls[0].obj );

			balls[1] = new MyCircle(dispX(x2), dispY(y2), R_BALL);
			this.LayoutRoot.Children.Add( balls[1].obj );
        }

		// ボタンを押したらリスタート
		private void Do_btn1_Click(object sender, RoutedEventArgs e)
		{
			initVars();
		}

        // タイマーで呼び出されるメインハンドラー
        private void MainHandler(object o, EventArgs sender)
        {
			// ここでボールの移動を行う
			for (int i = 0; i < 50; i++) // 細かくステップを刻む
			{
				UpdateAction();
			}

			// １個目のボール
			x1 = R1 * Math.Sin(TH[0]);
			y1 = -R1 * Math.Cos(TH[0]);
			// ２個目のボール
			x2 = x1 + R2 * Math.Sin(TH[1]);
			y2 = y1 - R2 * Math.Cos(TH[1]);

			// ボールの再描画
			balls[0].x = dispX(x1);
			balls[0].y = dispY(y1);
			balls[0].Action();

			balls[1].x = dispX(x2);
			balls[1].y = dispY(y2);
			balls[1].Action();

			// 線の描画
		//	lines[0].X1 = dispX(0.0);
		//	lines[0].Y1 = dispY(0.0);
			lines[0].X2 = dispX(x1);
			lines[0].Y2 = dispY(y1);

			lines[1].X1 = dispX(x1);
			lines[1].Y1 = dispY(y1);
			lines[1].X2 = dispX(x2);
			lines[1].Y2 = dispY(y2);

			// 数字の表示、動いていると何となくそれっぽい
			DebugText.Text
				= "(" + x1.ToString("F") + ", " + y1.ToString("F") + ")"
				+ " - (" + x2.ToString("F") + ", " + y2.ToString("F") + ")";
		}

		//// 動かす
		// 固定長配列がない！
		// 毎回newしてたら大変なので。
		double[] x = new double[2];
		double[] v = new double[2];
		double[] acc = new double[2];
		double[] k1v = new double[2];
		double[] k1x = new double[2];
		double[] k2x = new double[2];
		double[] k2v = new double[2];
		double[] k3x = new double[2];
		double[] k3v = new double[2];
		double[] k4x = new double[2];
		double[] k4v = new double[2];

		// ルンゲ・クッタ法で計算
		private void UpdateAction()
		{
			accel(TH, OM, acc);
			for (int i = 0; i < 2; i++)
			{
				k1v[i] = acc[i] * dt;
				k1x[i] = OM[i] * dt;
				x[i] = TH[i] + k1x[i] / 2;
				v[i] = OM[i] + k1v[i] / 2;
			}

			accel(TH, OM, acc);
			for (int i = 0; i < 2; i++)
			{
				k2v[i] = acc[i] * dt;
				k2x[i] = (OM[i] + k1v[i] / 2) * dt;
				x[i] = TH[i] + k2x[i] / 2;
				v[i] = OM[i] + k2v[i] / 2;
			}

			accel(TH, OM, acc);
			for (int i = 0; i < 2; i++)
			{
				k3v[i] = acc[i] * dt;
				k3x[i] = (OM[i] + k2v[i] / 2) * dt;
				x[i] = TH[i] + k3x[i] / 2;
				v[i] = OM[i] + k3v[i] / 2;
			}

			accel(TH, OM, acc);
			for (int i = 0; i < 2; i++)
			{
				k4v[i] = acc[i] * dt;
				k4x[i] = (OM[i] + k3v[i]) * dt;
				TH[i] = TH[i] + (k1x[i] + 2 * k2x[i] + 2 * k3x[i] + k4x[i]) / 6;
				OM[i] = OM[i] + (k1v[i] + 2 * k2v[i] + 2 * k3v[i] + k4v[i]) / 6;
			}
        }

		// 加速！
		private void accel(double[] x, double[] v, double[] acc)
		{
			double den;
			double c12, s12, f1, f2;

			c12 = Math.Cos(x[0] - x[1]);
			s12 = Math.Sin(x[0] - x[1]);
			f1 = -s12 * v[1] * v[1] - A * Math.Sin(x[0]);
			f2 = s12 * v[0] * v[0] - Math.Sin(x[1]);
			den = A * B - c12 * c12;
			acc[0] = (B * f1 - c12 * f2) / den;
			acc[1] = (A * f2 - c12 * f1) / den;
		}

		// 画面座標への変換
		private double dispX(double x0)
		{
			return ( (x0 + 2.0) * width / 4.0 );
		}
		private double dispY(double y0)
		{
			return ( (-y0 + 2.0) * height / 4.0 );
		}
	}

    /**
     *  ボールオブジェクト
     */
    class MyCircle
    {
        public Ellipse obj;
        public double r;
		public double x, y;

        public MyCircle(double x0, double y0, double r0)
        {
            this.obj = new Ellipse();
            this.r = r0;
            this.x = x0;
            this.y = y0;
			this.obj.Width  = 2 * r0;
			this.obj.Height = 2 * r0;

            this.setColor(Color.FromArgb(0xFF, 0xFF, 0x99, 0x00)); //オレンジ色
        }

        public void setColor(Color c)
        {
            SolidColorBrush brush = new SolidColorBrush(c);
            this.obj.Fill = brush;
        }

        public void Action()
        {
            this.obj.SetValue(Canvas.LeftProperty, this.x - this.r);
			this.obj.SetValue(Canvas.TopProperty,  this.y - this.r);
        }
    }
}
